@@ -3,6 +3,7 @@ import React from 'react';
 import { openMenu, select } from 'react-select-event';
 
 import { createMockTimeSeriesList } from '../__mocks__/cloudMonitoringQuery';
+import { MetricDescriptor } from '../types';
 
 import { GroupBy, Props } from './GroupBy';
 
@@ -12,7 +13,7 @@ const props: Props = {
   metricDescriptor: {
     valueType: '',
     metricKind: '',
-  } as any,
+  } as unknown as MetricDescriptor,
   variableOptionGroup: { options: [] },
   labels: [],
   query: createMockTimeSeriesList(),
